/********************************************************************************************
 *                                                                                          *
 * Copyright (C) 2017 Armin Felder, Dennis Beier                                            *
 * This file is part of RocketChatMobileEngine <https://git.fairkom.net/chat/fairchat>.     *
 *                                                                                          *
 * RocketChatMobileEngine is free software: you can redistribute it and/or modify           *
 * it under the terms of the GNU General Public License as published by                     *
 * the Free Software Foundation, either version 3 of the License, or                        *
 * (at your option) any later version.                                                      *
 *                                                                                          *
 * RocketChatMobileEngine is distributed in the hope that it will be useful,                *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of                           *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                            *
 * GNU General Public License for more details.                                             *
 *                                                                                          *
 * You should have received a copy of the GNU General Public License                        *
 * along with RocketChatMobileEngine. If not, see <http://www.gnu.org/licenses/>.           *
 *                                                                                          *
 ********************************************************************************************/


#ifndef ROCKETCHATMESSAGE_H
#define ROCKETCHATMESSAGE_H

#include <QJsonObject>
#include <QList>
#include <QSharedPointer>
#include "repos/entities/rocketchatattachment.h"
#include "repos/emojirepo.h"
#include "utils.h"
#include "rocketchatattachment.h"

class EmojiRepo;

class RocketChatMessage
{
    public:
        enum class Type {
            textmessage
        };
        RocketChatMessage( QJsonObject data, unsigned long timestamp, RocketChatMessage::Type messageType = Type::textmessage );
        RocketChatMessage( QJsonObject data );
        RocketChatMessage( QString roomId, QString message );
        bool operator > ( const RocketChatMessage &compMessage ) const
        {
            return ( timestamp > compMessage.getTimestamp() );
        }
        bool operator > ( const RocketChatMessage *compMessage ) const
        {
            if ( compMessage == nullptr ) {
                return true;
            }

            return ( timestamp > compMessage->getTimestamp() );
        }

        qint64 getTimestamp() const;
        bool isEmpty() const;
        QJsonObject getData() const;

        QString getRoomId() const;
        void setRoomId( const QString &value );

        QString getId() const;

        QString getAuthor() const;
        void setAuthor( const QString &value );

        QString getFormattedDate() const;
        void setFormattedDate( const QString &value );

        QString getFormattedTime() const;
        void setFormattedTime( const QString &value );

        QString getMessageString() const;
        void setMessageString( const QString &value );

        QString getMessageType();
        void setMessageType( const QString &value );
        bool getOwnMessage() const;
        void setOwnMessage( bool value );

        void emojify( EmojiRepo *pEmojiRepo );

        QList<QSharedPointer<RocketChatAttachment> > getAttachments() const;
        void setAttachments( const QList<QSharedPointer<RocketChatAttachment> > &value );



        qint64 getEmojiHash() const;
        void setEmojiHash( const qint64 &pValue );

        QString getAuthorId() const;
        void setAuthorId( const QString &value );

    protected:
        bool empty = false;
        qint64 timestamp = 0;
        QJsonObject data;
        QString roomId;
        Type type;
        bool ownMessage = false;
        QString originalMessage;
        QString parsedMessage;
        QString formattedTime;
        QString formattedDate;
        QString messageString;
        QString id;
        QString author;
        QString authorId;
        QString messageType;
        QList<QSharedPointer<RocketChatAttachment>> attachments;
        qint64 mEmojiHash = 0;
};

#endif // ROCKETCHATMESSAGE_H
